<?php

/**
 * @file
 * Contains \Drupal\otp_verification\Form\MiniorangeOTPVerificationCustomerSetup.
 */

namespace Drupal\otp_verification\Form;

use Drupal\otp_verification\MiniorangeOtpUtilities;
use Drupal\otp_verification\MiniorangeOTPVerificationCustomer;
use Drupal\Core\Form\FormBase;

class MiniorangeOTPVerificationCustomerSetup extends FormBase {

    public function getFormId() {
        return 'miniorange_otp_verification_customer_setup';
    }

    public function buildForm(array $form, \Drupal\Core\Form\FormStateInterface $form_state)
    {
        global $base_url;
        $form['markup_library'] = array(
            '#attached' => array(
                'library' => array(
                    "otp_verification/otp_verification.admin",
                    "otp_verification/otp_verification.module.css",
                    'otp_verification/otp_verification.phone',
                )
            ),
        );

        $current_status = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_status');

        $attachments['#attached']['library'][] = 'otp_verification/otp_verification.admin';

        if ($current_status == 'VALIDATE_OTP') {

            $form['markup_top_otp'] = array(
                '#markup' => '<div class="mo_otp_verification_table_layout_1"><div class="mo_otp_verification_table_layout mo_otp_container">'
            );

            $form['miniorange_otp_verification_customer_otp_token'] = array(
                '#type' => 'textfield',
                '#title' => t('OTP<span style="color: red">*</span>'),
            );

            $form['miniorange_otp_verification_customer_validate_otp_button'] = array(
                '#type' => 'submit',
                '#value' => t('Validate OTP'),
                '#submit' => array('::miniorange_otp_verification_validate_otp_submit'),
            );

            $form['miniorange_otp_verification_customer_setup_resendotp'] = array(
                '#type' => 'submit',
                '#value' => t('Resend OTP'),
                '#submit' => array('::miniorange_otp_verification_resend_otp'),
            );

            $form['miniorange_otp_verification_customer_setup_back'] = array(
                '#type' => 'submit',
                '#value' => t('Back'),
                '#submit' => array('::miniorange_otp_verification_back'),
            );
             MiniorangeOtpUtilities::AddSupportButton( $form, $form_state);

            $form['main_layout_div_end_1'] = array(
                '#markup' => '</div><div>',
            );
             MiniorangeOtpUtilities::Two_FA_Advertisement($form, $form_state);

            return $form;
        }
        elseif ($current_status == 'PLUGIN_CONFIGURATION')
        {

            $form['markup_top_message'] = array(
                '#markup' => '<div class="mo_otp_verification_table_layout_1"><div class="mo_otp_verification_table_layout mo_otp_container">'
            );

            $form['markupboit_message'] = array(
                '#markup' => '<div class="mo_otp_verification_welcome_message">Thank you for registering with miniOrange</div></br><h4>Your Profile: </h4>'
            );

            $header = array(
                'email' => array(
                    'data' => t('Customer Email')
                ),
                'customerid' => array(
                    'data' => t('Customer ID')
                ),
                'token' => array(
                    'data' => t('Token Key')
                ),
                'apikey' => array(
                    'data' => t('API Key')
                ),
            );

            $options = [];

            $options[0] = array(
                'email' => \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_email'),
                'customerid' => \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_id'),
                'token' => \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_token'),
                'apikey' => \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_api_key'),
            );

            $form['fieldset']['customerinfo'] = array(
                '#theme' => 'table',
                '#header' => $header,
                '#rows' => $options,
            );

            $form['markup_1'] = array(
                '#markup' => '<br><b>Track your Transactions:</b><br><br>'
                . 'Follow the steps to view your transactions: <br><br>'
                . '1. Click on the button below.<br>'
                . '2. Login using the credentials you used to register for this plugin.<br>'
                . '3. You will be presented with <b><i>View Transactions</i></b> page.<br>'
                . '4. From this page you can track your remaining transactions<br><br>',
            );
            $user_email = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_email');
            $form['markup_2'] = array(
                '#markup' => '<br><b>View Transactions:</b><br><br>'
                . '<a class="mbtn mo_otp-mbtn-primary mbtn-large" target="_blank" style="padding:5px; text-align:center;" href="https://login.xecurify.com/moas/login?username='.$user_email.'&redirectUrl=https://login.xecurify.com/moas/viewtransactions">View Transaction</a></h4>',
            );

            $form['main_layout_div_end'] = array(
                '#markup' => '<br><br></div>',
            );

            MiniorangeOtpUtilities::AddSupportButton( $form, $form_state);
            $form['main_layout_div_end_1'] = array(
                '#markup' => '<div>',
            );
            MiniorangeOtpUtilities::Two_FA_Advertisement($form, $form_state);

            return $form;
        }

        $form['header_top_style_1'] = array('#markup' => '<div class="mo_otp_verification_table_layout_1">',);

        $form['markup_top'] = array(
            '#markup' => '<div class="mo_otp_verification_table_layout mo_otp_container">'
        );

        $form['markup_14'] = array('#markup' => '<h3>Register/login with miniOrange</h3><hr><br>');

        $form['markup_15'] = array(
            '#markup' => 'Just complete the short registration below to configure' . ' the OTP Verification module. Please enter a valid email id <br>that you have' . ' access to. You will be able to move forward after verifying an OTP' . ' that we will send to this email.'
        );

        $form['miniorange_otp_verification_customer_setup_username'] = array(
            '#type' => 'textfield',
            '#title' => t('Email<span style="color: red">*</span>'),
        );

        $form['miniorange_otp_verification_customer_setup_phone'] = array(
            '#type' => 'textfield',
            '#title' => t('Phone'),
            '#id'=>'query_phone',
            '#attributes'=>array('class'=>array('query_phone'),),
            '#description' => '<b>Note:</b> We will only call if you need support.',
        );

        $form['miniorange_otp_verification_customer_setup_password'] = array(
            '#type' => 'password_confirm',
        );

        $form['miniorange_otp_verification_customer_setup_button'] = array(
          '#type' => 'submit',
          '#value' => t('Register'),
        );

        MiniorangeOtpUtilities::AddSupportButton( $form, $form_state);
        $form['main_layout_div_end_1'] = array(
                '#markup' => '</div>',
            );
         MiniorangeOtpUtilities::Two_FA_Advertisement($form, $form_state);

        return $form;
    }

    public function submitForm(array &$form, \Drupal\Core\Form\FormStateInterface $form_state) {
        $username = $form['miniorange_otp_verification_customer_setup_username']['#value'];
        $phone = $form['miniorange_otp_verification_customer_setup_phone']['#value'];
        $password = $form['miniorange_otp_verification_customer_setup_password']['#value']['pass1'];
        if(empty($username)||empty($password)){
            \Drupal::messenger()->addMessage(t('The <b><u>Email </u></b> and <b><u>Password</u></b> fields are mandatory.'), 'error');
            return;
        }
        if (!\Drupal::service('email.validator')->isValid($username)) {
            \Drupal::messenger()->addMessage(t('The email address <i>' . $username . '</i> is not valid.'), 'error');
            return;
        }
        $customer_config = new MiniorangeOTPVerificationCustomer($username, $phone, $password, NULL);
        $check_customer_response = json_decode($customer_config->checkCustomer());
        if ($check_customer_response->status == 'CUSTOMER_NOT_FOUND') {
            \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_email', $username)->save();
            \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_phone', $phone)->save();
            \Drupal::configFactory()->getEditable('miniorange_otp_verification_idp.settings')->set('miniorange_otp_verification_customer_admin_password', $password)->save();
            $send_otp_response = json_decode($customer_config->sendOtp());

            if ($send_otp_response->status == 'SUCCESS') {
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_tx_id', $send_otp_response->txId)->save();
                $current_status = 'VALIDATE_OTP';
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_status', $current_status)->save();
                \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode sent to @username', ['@username' => $username]),'status');
            }
        }
        elseif ($check_customer_response->status == 'CURL_ERROR') {
            \Drupal::messenger()->addMessage(t('cURL is not enabled. Please enable cURL'), 'error');
        }
        else {
            $customer_keys_response = json_decode($customer_config->getCustomerKeys());
            if (json_last_error() == JSON_ERROR_NONE) {
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_id', $customer_keys_response->id)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_token', $customer_keys_response->token)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_email', $username)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_phone', $phone)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_api_key', $customer_keys_response->apiKey)->save();
                $current_status = 'PLUGIN_CONFIGURATION';
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_status', $current_status)->save();
                \Drupal::messenger()->addMessage(t('Successfully retrieved your account.'), 'status');
            }
            else {
                \Drupal::messenger()->addMessage(t('Invalid credentials'), 'error');
            }
        }
    }

    public function miniorange_otp_verification_back(&$form, $form_state) {
        $current_status = 'CUSTOMER_SETUP';
        \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_status', $current_status)->save();
        \Drupal::configFactory()->getEditable('otp_verification.settings')->clear('miniorange_miniorange_otp_verification_customer_admin_email')->save();
        \Drupal::configFactory()->getEditable('otp_verification.settings')->clear('miniorange_otp_verification_customer_admin_phone')->save();
        \Drupal::configFactory()->getEditable('otp_verification.settings')->clear('miniorange_otp_verification_tx_id')->save();
        \Drupal::messenger()->addMessage(t('Register/Login with your miniOrange Account'),'status');
    }

    public function miniorange_otp_verification_resend_otp(&$form, $form_state) {
        \Drupal::configFactory()->getEditable('otp_verification.settings')->clear('miniorange_otp_verification_tx_id')->save();
        $username = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_email');
        $phone = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_phone');
        $customer_config = new MiniorangeOTPVerificationCustomer($username, $phone, NULL, NULL);
        $send_otp_response = json_decode($customer_config->sendOtp());
        if ($send_otp_response->status == 'SUCCESS') {
            // Store txID.
            \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_tx_id', $send_otp_response->txId)->save();
            $current_status = 'VALIDATE_OTP';
            \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_status', $current_status)->save();
            \Drupal::messenger()->addMessage(t('Verify email address by entering the passcode sent to @username', array('@username' => $username)),'status');
        }
    }

    public function miniorange_otp_verification_validate_otp_submit(&$form, $form_state) {
        $otp_token = $form['miniorange_otp_verification_customer_otp_token']['#value'];
        $otp_token = trim($otp_token);
        $username = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_email');
        $phone = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_phone');
        $tx_id = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_tx_id');
        $customer_config = new MiniorangeOTPVerificationCustomer($username, $phone, NULL, $otp_token);
        $validate_otp_response = json_decode($customer_config->validateOtp($tx_id));

        if ($validate_otp_response->status == 'SUCCESS')
        {
            \Drupal::configFactory()->getEditable('otp_verification.settings')->clear('miniorange_otp_verification_tx_id')->save();
            $password = \Drupal::config('otp_verification.settings')->get('miniorange_otp_verification_customer_admin_password');
            $customer_config = new MiniorangeOTPVerificationCustomer($username, $phone, $password, NULL);
            $create_customer_response = json_decode($customer_config->createCustomer());
            if ($create_customer_response->status == 'SUCCESS') {
                $current_status = 'PLUGIN_CONFIGURATION';
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_status', $current_status)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_email', $username)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_phone', $phone)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_admin_token', $create_customer_response->token)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_id', $create_customer_response->id)->save();
                \Drupal::configFactory()->getEditable('otp_verification.settings')->set('miniorange_otp_verification_customer_api_key', $create_customer_response->apiKey)->save();
                \Drupal::messenger()->addMessage(t('Customer account created.'), 'status');
            }
            else {
                \Drupal::messenger()->addMessage(t('Error creating customer'), 'error');
            }
        }
        else {
            \Drupal::messenger()->addMessage(t('Error validating OTP'), 'error');
        }
    }
}
